SlabIndex

Struct SlabIndex 

pub struct SlabIndex(/* private fields */);
Transparent type that represents an index into a Slab.

used to discourage accessing the Slab at arbitrary indexes.

Implementations

Source

impl SlabIndex

pub fn i_actually_really_know_what_i_am_doing_and_i_want_the_inner_usize( &self, ) -> usize

Returns the inner usize.

Annoyingly long names discourage use and make you really think about what you are doing.

pub fn i_actually_really_know_what_i_am_doing_and_i_want_to_construct_from_usize( i: usize, ) -> Self

Returns a new SlabIndex created from the provided usize. Annoyingly long names discourage use and make you really think about what you are doing.