macro_rules! wire {
($g:expr,$name:ident) => { ... };
}Expand description
Creates a wire with the same variable name and gate name.
macro_rules! wire {
($g:expr,$name:ident) => { ... };
}Creates a wire with the same variable name and gate name.