pub struct OutputHandle(/* private fields */);Expand description
Handle type that represents a set of gates in an InitializedGateGraph or GateGraphBuilder which we want to query.
Implementations
Sourceimpl OutputHandle
impl OutputHandle
Sourcepub fn u8(self, g: &InitializedGateGraph) -> u8
pub fn u8(self, g: &InitializedGateGraph) -> u8
Returns a value of the corresponding type created from the current state bits in the output.
If there are more bits than size_of::<type>, the excess bits will be ignored.
If there are less bits, the value will be 0 extended.
Sourcepub fn print_u8(self, g: &InitializedGateGraph)
pub fn print_u8(self, g: &InitializedGateGraph)
Prints the output of the corresponding type() function along with the name of the output.
Sourcepub fn i8(self, g: &InitializedGateGraph) -> i8
pub fn i8(self, g: &InitializedGateGraph) -> i8
Returns a value of the corresponding type created from the current state bits in the output.
If there are more bits than size_of::<type>, the excess bits will be ignored.
If there are less bits, the value will be 0 extended.
Sourcepub fn print_i8(self, g: &InitializedGateGraph)
pub fn print_i8(self, g: &InitializedGateGraph)
Prints the output of the corresponding type() function along with the name of the output.
Sourcepub fn u16(self, g: &InitializedGateGraph) -> u16
pub fn u16(self, g: &InitializedGateGraph) -> u16
Returns a value of the corresponding type created from the current state bits in the output.
If there are more bits than size_of::<type>, the excess bits will be ignored.
If there are less bits, the value will be 0 extended.
Sourcepub fn print_u16(self, g: &InitializedGateGraph)
pub fn print_u16(self, g: &InitializedGateGraph)
Prints the output of the corresponding type() function along with the name of the output.
Sourcepub fn i16(self, g: &InitializedGateGraph) -> i16
pub fn i16(self, g: &InitializedGateGraph) -> i16
Returns a value of the corresponding type created from the current state bits in the output.
If there are more bits than size_of::<type>, the excess bits will be ignored.
If there are less bits, the value will be 0 extended.
Sourcepub fn print_i16(self, g: &InitializedGateGraph)
pub fn print_i16(self, g: &InitializedGateGraph)
Prints the output of the corresponding type() function along with the name of the output.
Sourcepub fn u32(self, g: &InitializedGateGraph) -> u32
pub fn u32(self, g: &InitializedGateGraph) -> u32
Returns a value of the corresponding type created from the current state bits in the output.
If there are more bits than size_of::<type>, the excess bits will be ignored.
If there are less bits, the value will be 0 extended.
Sourcepub fn print_u32(self, g: &InitializedGateGraph)
pub fn print_u32(self, g: &InitializedGateGraph)
Prints the output of the corresponding type() function along with the name of the output.
Sourcepub fn i32(self, g: &InitializedGateGraph) -> i32
pub fn i32(self, g: &InitializedGateGraph) -> i32
Returns a value of the corresponding type created from the current state bits in the output.
If there are more bits than size_of::<type>, the excess bits will be ignored.
If there are less bits, the value will be 0 extended.
Sourcepub fn print_i32(self, g: &InitializedGateGraph)
pub fn print_i32(self, g: &InitializedGateGraph)
Prints the output of the corresponding type() function along with the name of the output.
Sourcepub fn u64(self, g: &InitializedGateGraph) -> u64
pub fn u64(self, g: &InitializedGateGraph) -> u64
Returns a value of the corresponding type created from the current state bits in the output.
If there are more bits than size_of::<type>, the excess bits will be ignored.
If there are less bits, the value will be 0 extended.
Sourcepub fn print_u64(self, g: &InitializedGateGraph)
pub fn print_u64(self, g: &InitializedGateGraph)
Prints the output of the corresponding type() function along with the name of the output.
Sourcepub fn i64(self, g: &InitializedGateGraph) -> i64
pub fn i64(self, g: &InitializedGateGraph) -> i64
Returns a value of the corresponding type created from the current state bits in the output.
If there are more bits than size_of::<type>, the excess bits will be ignored.
If there are less bits, the value will be 0 extended.
Sourcepub fn print_i64(self, g: &InitializedGateGraph)
pub fn print_i64(self, g: &InitializedGateGraph)
Prints the output of the corresponding type() function along with the name of the output.
Sourcepub fn u128(self, g: &InitializedGateGraph) -> u128
pub fn u128(self, g: &InitializedGateGraph) -> u128
Returns a value of the corresponding type created from the current state bits in the output.
If there are more bits than size_of::<type>, the excess bits will be ignored.
If there are less bits, the value will be 0 extended.
Sourcepub fn print_u128(self, g: &InitializedGateGraph)
pub fn print_u128(self, g: &InitializedGateGraph)
Prints the output of the corresponding type() function along with the name of the output.
Sourcepub fn i128(self, g: &InitializedGateGraph) -> i128
pub fn i128(self, g: &InitializedGateGraph) -> i128
Returns a value of the corresponding type created from the current state bits in the output.
If there are more bits than size_of::<type>, the excess bits will be ignored.
If there are less bits, the value will be 0 extended.
Sourcepub fn print_i128(self, g: &InitializedGateGraph)
pub fn print_i128(self, g: &InitializedGateGraph)
Prints the output of the corresponding type() function along with the name of the output.
Sourcepub fn char(self, g: &InitializedGateGraph) -> char
pub fn char(self, g: &InitializedGateGraph) -> char
Returns a value of the corresponding type created from the current state bits in the output.
If there are more bits than size_of::<type>, the excess bits will be ignored.
If there are less bits, the value will be 0 extended.
Sourcepub fn print_char(self, g: &InitializedGateGraph)
pub fn print_char(self, g: &InitializedGateGraph)
Prints the output of the corresponding type() function along with the name of the output.
pub fn bx(self, g: &InitializedGateGraph, n: usize) -> bool
pub fn b0(&self, g: &InitializedGateGraph) -> bool